function A = get_LP(F,LP)
% creates equality constraint matrix
% LP can be 'BLP' or 'FLP'
% first variable is mu0 = 1
%
if ~exist('LP','var') || isempty(LP)
	LP = 'FLP';
end

nV = get_nV(F);
nA = get_nA(F);
H = F.H;

% how many relaxed variables - as many as function values
i = 1;
mu = double(F.H.A>0);
n = nnz(mu);
mu(mu>0) = i:i+n-1;
i = i+n;
const = mu(1);
%dummy  = i;
nvars = i-1;

A = sparse(0,nvars);

% marginalization constraints
for a=2:nA
	d = H.d(a);
	ii = [];
	jj = [];
	vv = [];
	eq = 0;
	if strcmp(LP,'FLP')
		% for subsets of order 1 less than a (should be d such subsets)
		% for all subset configurations
		% how many constraints: d* 2^(d-1)
		% 2 summands in each constraint
		for p = 1:d % for all subsets of order d-1
			z = uint32(2^d-1);
			z = bitset(z,p,0);
			%z = bitset(z,d-p+1,0);
			b = H.A(z+1,a);
			% loop over configurations
			for z = uint32(0:2^d-1) % configuration of the subset and of the varied bit
				x = mdec2bin(z,d);
				% configuration of the subset alone
				sx = x;
				%sx(p) = [];
				sx(d-p+1) = [];
				sz  = mbin2dec(sx);
				% flip the p'th bit in x
				z1 = bitxor(z,uint32(2^(p-1)));
				% constraint: mu(x,a)+mu(x1,a)-mu(sx,b) = 0
				% calculate constraint number
				i = (2^(d-1))*(p-1) + sz;
				%eq = eq+1;
				sub = [i*3+1:(i+1)*3];
				ii(sub) = i+1;
				jj(sub) = [mu(z+1,a) mu(z1+1,a) mu(sz+1,b)];
				vv(sub) = [1 1 -1];
			end
		end
	elseif strcmp(LP,'BLP')
		% normalization separately
		if d==1
			ii = [1 1 1];
			jj = [mu(1,a) mu(2,a) mu(1,1)];
			vv = [1 1 -1];
		else
			% for all nodes
			% how many constraints: d*2
			% 2^(d-1) summands in each constraint
			for p = 1:d % for all nodes
				z = 2^(p-1); % where is the node p
				b = H.A(z+1,a);
				% loop over configurations
				for z = uint32(0:2^d-1) % configuration of the subset and of the varied bit
					x = mdec2bin(z,d);
					% configuration of the subset alone
					sx = x(d-p+1);
					sz  = mbin2dec(sx);
					% % flip the p'th bit in x
					% % z1 = bitxor(z,uint32(2^(p-1)));
					% constraint: sum_{x | x(p)=sx} mu(x,a)-mu(sx,b) = 0
					% calculate constraint number
					i = 2*(p-1) + sz + 1; % constr index
					% how many non-zeor terms: 2^(d-1) + 1
					n = 2^(d-1) + 1;
					Sx = x;
					Sx(d-p+1) = [];
					Sz  = mbin2dec(Sx);
					j = Sz+1; % entry index
					k = (i-1)*n+j;
					ii(k) = i;
					jj(k) = mu(z+1,a);
					vv(k) = 1;
					%sub = [i*n+1:(i+1)*n];
					%ii(sub) = i+1; % constraint index
					%jj(sub) = [mu(z+1,a) mu(z1+1,a) mu(sz+1,b)];
					%vv(sub) = [1 1 -1];
				end
				sz = [0 1];
				i = 2*(p-1) + sz + 1;
				j = 2^(d-1) + 1;
				k = (i-1)*n+j;
				ii(k) = i;
				jj(k) = mu(sz+1,b);
				vv(k) = -1;
			end
		end
	else
		error('wrong type');
	end
	A1 = sparse(ii,jj,vv,max(ii),nvars);
	A = [A; A1];
end

if 0 % print matrix
	[mui muj] = find(mu);
	for i = 1:size(A,1)
		[ans j v] = find(A(i,:));
		for l=1:length(j)
			% find hyperedge and labeling back from index j
			a = muj(j(l));
			v = H.get_nodes(a);
			d = length(v);
			z = mui(j(l))-1;
			x = fliplr(dec2bin(z,d));
			if A(i,j(l))==1
				fprintf('+');
			elseif A(i,j(l))==-1
				fprintf('-');
			else
				fprintf(num2str(A(i,j(l))));
			end
			if isempty(v)
				fprintf('1');
			else
				if length(v)==1
					sv = num2str(v);
				else
					sv = [sprintf('%i,',v(1:end-1)) num2str(v(end))];
				end
				fprintf('mu_%s(%s)',sv,x);
			end
			%mu(i,j)
		end
		fprintf(' = 0\n');
	end
end

end