function [E1 pi] = restrict(E,p)
% [E1 pi] = restrict(E,p)
% takes energy E and applies map p to it, excluding the assigned variables
% E1 - restricted energy on a smaller hypergraph,
% pi - injection of node numbers from E1 to E
%
nV = get_nV(E);
nA = get_nA(E);
H = E.H;

mask = p(1,:)==p(2,:); % where variables are fixed
pi = find(~mask);
nV1 = nnz(~mask);
maxd1 = H.maxd;
ip = zeros(1,nV);
ip(~mask) = 1:nV1; % map node numbers from E1 to E

H1 = hyper_graph1(nV1,maxd1);
E1 = hyper_energy1(H1);
E1.fA(:) = 0;
for a = 1:nA % over all hyperedges of A
	d = H.d(a);
	v = fliplr(H.get_nodes(a)); % nodes of a in H
	v1 = ip(v(~mask(v))); % nodes remaining in H1
	[H1 a1] = H1.add_edge(v1); % add or find this hyperedge in H1
	if a1>size(E1.fA,2)
		E1.fA(:,a1) = 0;
	end
	% add cost of a to a1 for all configurations
	m = mask(v); % configuration bits that are fixed
	y = p(1,v(m)); % assignment of the fixed bits
	d1 = d-nnz(m);
	for z1 = 0:2^d1-1 % over configurations that are still free
		x1 = mdec2bin(z1,d1);
		x = zeros(1,d);
		x(m) = y;
		x(~m) = x1; % completed configuration on a
		z = mbin2dec(x);
 		E1.fA(z1+1,a1) = E1.fA(z1+1,a1)+E.fA(z+1,a); % aggregate cost there
	end
end
E1.H = H1;
end