function [Z] = get_Zeta(H)
% Construct inequalities of the Sherali-Adams relaxation in the form Z*zeta >=0
% Assumes constraint zeta_0 = 1
%

% activate debug to print out inequalities
debug =  0;

nA = H.get_nA();
% how many zeta? = nA
Z = sparse(0,nA);

eq = 0; % count inequalities in Z
% constraints: forall c\in \E forall a\subset c
% \sum_{d | (not a)\subset d, d \subset c} (-1)^|a cap d| zeta_d
% same as
% \sum_{b | \subset a} (-1)^|b| zeta_{ b \cup (not a) }
% mask to exclude hyperedges that are proper subsets of other hyperedges
amask = true(1,nA);
% loop in decreasing degree
for dc = H.maxd:-1:0
	% loop over hyperedges if degree dc
	for c = find(H.d==dc)
		if(~amask(1,c)) % skip hyperedges that were processed as subsets of other hyperedges
			continue;
		end
		%for c=1:nA
		% nodes of c
		dc = H.d(c);
		vc = H.get_nodes(c);
		% loop over subsets
		for za = 0:2^dc-1
			% start new constraint
			eq = eq+1;
			% subset a
			a = H.A(za+1,c);
			% deactivate hyperedge a in amask
			amask(1,a) = 0;
			% nodes of a
			% calculate entry of P_zeta_{c,x_c,x'_c} with x_c' = 1_c, x_c ~ a
			% loop over b \subset a
			xa = mdec2bin(za,dc); % subset mask of a
			% ~xa must be in d
			% how many bits in a
			na = nnz(xa);
			for zb=0:2^na-1
				xb = mdec2bin(zb,na); % bits of b
				sgn = (-1)^nnz(xb);
				% calculate subset d = (c\a)\cup b
				xd = ~xa;
				if ~isempty(xb)
					xd(xa) = xd(xa) | xb;
				end
				zd = mbin2dec(xd);
				d = H.A(zd+1,c);
				Z(eq,d) = sgn; % encoding constraint Z zeta >= 0
				% subtract in Q the term sgn*f_c(x_c) from the entry (c,x_c),d
				%mui = I(za+1,c);
				%PZ(mui,d) = PZ(mui,d)+f(mui)*sgn;
				if debug
					vd = H.get_nodes(d);
					if isempty(vd)
						fprintf('1 ');
					else
						s = strrep(num2str(vd),'  ',',');
						if sgn>0
							fprintf('+');
						else
							fprintf('-');
						end
						fprintf('z_%s ',s);
					end
				end
			end
			if debug
				fprintf(' >=0\n');
			end
		end
	end
end
end