% This code implements experiments described in
% A. Shekhovtsov: "Higher Order Maximum Persistency and Comparison Theorems"
% for higher order models.
% The goal is to allow reproducibility of the results and make the
% reference implementation of the method available to other researchers for
% possible other comparisons
%
% Execution of methods HOCR, Fix, GRD* depends on the submodular library
% v.1.0 by P. Strandmark and dependencies, see https://github.com/PetterS/submodular
% Follow their instructions and add path to /submodular/matlab
%
% Execution of Methods Swoboda-14 and L1-FLP depends on IBM ILOG CPLEX,
% (available for academic usage)
% add path to cplex/matlab/your_platform
%
% by Alexander Shekhovtsov (shekhovt@icg.tugraz.at), 2015
%

commandwindow;
run ../mtools/mpath.m
maddpath('../minsum');
%addpath('Z:\work\poh\submodular\matlab\');

% path for storing experiment results
pth = 'exp-randt-h/';

% select methods to run
methods = {'HOCR','Fix','GRD-heur','GRD','GRD-gen','Swoboda-14','L1-FLP'};
%methods = {'HOCR','Fix','GRD-heur','GRD','GRD-gen','Swoboda-14','L1-FLP','L1-FLP','L1-Iterative-FLP'};

% select methods which have to be executed even if their result already exists
% otherwise continues with not yet computed instances (continue after interrupting)
renew = {};
%renew = {'HOCR','Fix','GRD-heur','GRD','GRD-gen','Swoboda-14','L1-FLP'};

% number of labels, must be 2
K = 2;

% how many runs per parameter setting
firstit = 1;
maxit = 100;

% experiment type, select one of the lines
%___random polynomials
%NN = 100; MM = 1; maxd = 3; T = 100; gen = 'poly_KS'; tight = 'any';
%NN = 100; MM = 1; maxd = 3; T = 200; gen = 'poly_KS'; tight = 'FLP-tight';
%NN = 100; MM = 1; maxd = 4; T = 30; gen = 'poly_KS'; tight = 'any';
%NN = 100; MM = 1; maxd = 4; T = 50; gen = 'poly_KS'; tight = 'FLP-tight';
%___random problems on the grid 
NN = 10; MM = NN; maxd = 3; T = 0; gen = 'grid'; tight = 'FLP-tight';
%NN = 10; MM = NN; maxd = 4; T = 0; gen = 'grid'; tight = 'FLP-tight';
%NN = 20; MM = NN; maxd = 3; T = 4; gen = 'grid_KS'; tight = 'any';
%___scaling experiment
%NN = [5 10 15 20 30 40 50 60]; MM = NN; maxd = 3; T = 0; gen = 'grid'; tight = 'FLP-tight';maxit = 10;pth = 'exp-randt-ht/';

LP = 'FLP'; % type of the relaxation
%LP = 'BLP'; % experimental
record = true; % write down files

if length(NN)>1
	figure(1);figure(2);figure(4);
end

figure(3);

rmethods = {};
for mth = 1:length(methods)
	method = strrep(methods{mth},'-','_');
	method = strrep(method,'.','_');
	method = strrep(method,' ','_');
	rmethods{mth} = method;
end

for Ni = 1:length(NN) % loop over problem sizes
	N = NN(Ni);
	M = MM(Ni);
	
	nV = N*M;
	%gen = 'grid';
	%tight = 'any'; % 'BLP-tight', 'FLP-tight'
	%tight = 'FLP-tight'; % 'BLP-tight',
	
	maxelim = nV;
	
	%ptype = 'test1';
	%ptype = ['N' num2str(N) '_M' num2str(M) '_d' num2str(maxd) '_' gen '_' LP '_' tight];
	ptype = ['N' num2str(N) '_M' num2str(M) '_d' num2str(maxd) '_' gen '_' tight];
	
	if strcmp(gen,'poly_KS') || strcmp(gen,'grid_KS')
		ptype = [ptype '_T' num2str(T)];
	end
	
	instfile = ['exp-rand-h/untight_seeds_ho_' ptype];
	out = [pth 'collect_' ptype '_' sprintf('N%2.2i',N)];
	
	% load what was computed before
	if 1
		try
			load(out);
			collect = experiment.collect;
			%firstit = experiment.it;
		catch e
			collect = [];
			%collect.legend = methods;
		end
	else
		if firstit==1
			collect = [];
			%collect.legend = methods;
		end
	end
	
	% generate non-tight instance
	
	try
		load(instfile);
	catch
		instances = {};
	end
	for it = firstit:maxit
		fprintf('------------------------it %i -------------------------\n',it);
		% pick up untight instances
		isz = size(instances);
		if(K>isz(1) || N>isz(2) || it > length(instances{K,N}))
			fprintf('generating new instance\n');
			if(K<=isz(1) && N<=isz(2) && ~isempty(instances{K,N}))
				seed0 = instances{K,N}(end);
			else
				instances{K,N} = [];
				seed0 = 0;
			end
			for iit=seed0+1:seed0+10000
				rand('seed',iit);
				randn('seed',iit);
				%E = hyper_energy1(H);
				E = hpo_gen(N,M,maxd,T,gen,iit);
				if strcmp(tight,'any')
					break;
				elseif strcmp(tight,'BLP-tight')
					[y LB mu1 Mu] = E.full_LP(false,'BLP');
				elseif strcmp(tight,'FLP-tight')
					[y LB mu1 Mu] = E.full_LP(false,'FLP');
				else
					error('bad');
				end
				f = E.fA(E.H.A>0);
				nnint = sum(sum(mu1.^2,1)<1-1e-3);
				if(nnint == 0)
					assert(E.cost(y)<LB+1e-5);
					fprintf('#%i is tight\n',iit);
					continue;
				end
				break;
			end
			instances{K,N}(end+1) = iit;
			force_path(instfile);
			save(instfile,'instances');
		end
		%
		seed = instances{K,N}(it);
		fprintf('Instance K=%i N=%i, #%i\n',K,N,seed);
		% check if has not solved already
		solved = true;
		if length(collect)<it
			collect(it).results = struct();
			solved = false;
		else
			for mth = 1:length(methods)
				method = methods{mth};
				rmethod = rmethods{mth};
				%mm = fieldnames(collect(it).results);
				if ~isfield(collect(it).results,rmethod) || ~isempty(strmatch(method,renew,'exact'))%~strmatch(rmethod, mm)
					solved = false;
					break;
				end
			end
		end
		if solved
			fprintf('all results exist\n');
		else
			rand('seed',seed);
			randn('seed',seed);
			E = hpo_gen(N,M,maxd,T,gen,seed);
			% solve relaxation
			[y LB mu1 Mu phi duals lp_time] = E.full_LP(false,'FLP');

			% run all methods
			p_L1 = [];
			for mth = 1:length(methods)
				method = methods{mth};
				rmethod = rmethods{mth};
				fprintf('____________%s_________________\n', method);				
				if ~isempty(strmatch(method,renew,'exact'))
					fprintf('renewing result\n');
				elseif isfield(collect(it),'results')  && isfield(collect(it).results,rmethod)
					fprintf('result exists\n');
					continue;
				end
				UB = 0;
				% execute and verify correctness of the method
				if ~isempty(strfind(method,'BLP'))
					[elim p time ilp_time ilp_reduced_time] = invoke_pho_method(method,E,[]);
				else
					[elim p time ilp_time ilp_reduced_time] = invoke_pho_method(method,E,y);
				end
				if ~isempty(strfind(method,'L1-FLP'))
					p_L1 = p;
				end
				collect(it).results.(rmethod) = struct;
				collect(it).results.(rmethod).time = time;
				collect(it).results.(rmethod).elim = elim;
				collect(it).results.(rmethod).ilp_time = ilp_time;
				collect(it).results.(rmethod).ilp_reduced_time = ilp_reduced_time;
				collect(it).results.(rmethod).p = p;
				if ~isempty(strfind(rmethod,'L1')) || ~isempty(strfind(rmethod,'Swoboda'))
					collect(it).results.(rmethod).lp_time = lp_time;
				else
					%collect(it).results.(rmethod).lp_time = 0;
				end
			end
			% compare persistency results
			p_L1 = collect(it).results.('L1_FLP').p;
			for mth = 1:length(methods)
				method = methods{mth};
				if strmatch(method,'L1-FLP')
					continue;
				end
				rmethod = rmethods{mth};
				p = collect(it).results.(rmethod).p;
				% compose p_max o p
				q = compose(p_L1+1,p+1)-1;
				% check p_L1 is better equal p
				% not guaranteed for week persistency, but we can try
				if ~all(q(:)==p_L1(:))
					fprintf('Failed test: mapping L1-FLP >= mapping %s\n',method);
					% must be that y was selected poorly
					y1 = mselect(p,y+1,1:nV);
					if E.cost(y1) < E.cost(y)
						fprintf('Rechecking with a better test labeling (not for record) y\n');
						[elim p1]  = invoke_pho_method('L1-FLP',E,y1);
						q = compose(p1+1,p+1)-1;
						assert(all(q(:)==p1(:)));
					else
						error('check me');
					end
					
				else
					fprintf('mapping L1-FLP >= mapping %s\n',method);
				end
			end
		end
		% display persistency and times
		exp2_bars_persistency;
		if(length(NN)>1)
			exp2_bars_time;
		end
		% write results to the file
		if record
			experiment = [];
			experiment.collect = collect;
			experiment.ptype = ptype;
			experiment.instances = instances{K,N}(1:it);
			experiment.K = K;
			experiment.N = N;
			experiment.M = M;
			experiment.nV = nV;
			experiment.maxelim = maxelim;
			experiment.T = T;
			experiment.gen = gen;
			experiment.maxd = maxd;
			experiment.tight = tight;
			%experiment.methods = methods;
			force_path(out);
			save(out,'experiment','nV','maxelim');
		end
		% plot time scaling
		if length(NN)>1
			save all
			ho_plot_time_scaling;
			load all
		end
	end
end