function [p time] = hpo_L1(E,y,LP)

nV = get_nV(E);
nA = get_nA(E);
H = E.H;
I = H.A;
nI = nnz(I);
I(I>0) = 1:nI;

if(~exist('y','var') || isempty(y))
	[y LB mu1 Mu] = E.full_LP(false,LP);
end

mask = y==1;
E1 = E.flip_energy(mask); % want that y becomes flipped to 0
E1 = E1.to_multilinear(); % reparametrize to a miltilinear polynomial (unique)
% now all components with at least one x_i=0 are zero
% create linear program
A0 = E1.get_LP(LP);
%nvars = size(A,2);
%const = 1;
mu = double(E.H.A>0);
n = nnz(mu);
mu(mu>0) = 1:n;
% A is my LP relaxation matrix
%
Z = H.get_Zeta(); % constraint Z zeta >=0, zeta_0 =  1 
%
% enumerate variables in L1
i = 1;
n = size(A0,1);  phi = i:i+n-1; i = i+n; % dual multipliers <-> primal constraints
n = nA; zeta = i:i+n-1; i = i+n; % variables zeta
const = zeta(1); % constant term = 1
nvars = i-1;

% variable limits
lb = -inf(nvars,1);
ub = inf(nvars,1);
lb(zeta) = 0;
ub(zeta) = 1;
lb(const) = 1; ub(const) = 1;

% test if better performance if reuse the dual solution from phase 1
% pretend we have the dual solution computed as follows
%[x_x x_LB x_mu1 x_Mu x_phi duals] = E1.full_LP(false,LP);
%f = E1.fA(H.A>0);
%df = A0'*duals;
%f = f - df; % apply equivalent transform
%E1.fA(E1.H.A>0) = f;

% system of constraints
% f'-fPZ*zeta - A' \varphi >= 0
% Z zeta >= 0
% what is the size of Q = -f'z0+fPZ: nI x nA, with 2*nA non-zeros
Q = sparse(nI,nA,2*nA);
for a=2:nA
	d = H.d(a);
	z = 2^d-1;
	% encode (I-P_\zeta')f_{a,x_a} for x_a = 1_a as (1-zeta_a)f_{a,x_a}
	Q(I(z+1,a),a) = -E1.fA(z+1,a);
	Q(I(z+1,a),1) = E1.fA(z+1,a);
end
%Q(:,1) = E.fA(H.A>0); % encode in Q the identity part, assumng zeta_0 = 1
% represent everything as constraint matrix AA [phi zeta] <= 0
AA = [A0' -Q; % encodes A0'\varphi - Q zeta <= 0 
	sparse(size(Z,1),size(A0',2)) -Z]; %encodes - Z zeta <= 0
bb = zeros(size(AA,1),1);
%
% test program is feasible
sol = zeros(nvars,1);
sol(zeta) = 1; % initial solution eliminate none
assert(max(AA*sol-bb)<=0);
%

% objective
cc = zeros(1,nvars); 
cc(zeta) = 0; % minimize \sum_a \zeta_a
cc(zeta(1:nV+1)) = 1; % minimize \sum_s \zeta_s
% cc(zeta) = 0; % minimize \sum_a \zeta_a
tic
if 0
	% with binary constraints for verification
	ops = cplexoptimset('lpmethod',0,'diagnostics','off','timelimit',1000); % automatic, 10 min time limit
	ctype = repmat('C',1,nvars);
	ctype(zeta) = 'B';
	[sol,fval,exitflag] = cplexmilp(cc,AA,bb,[],[],[],[],[],lb,ub,ctype,[],ops);
else
	% solve LP % CPLEX
	% 0 CPX_ALG_AUTOMATIC Automatic: let CPLEX choose; default
	% 1 CCPX_ALG_PRIMAL Primal simplex
	% 2 CPX_ALG_DUAL Dual simplex
	% 3 CPX_ALG_NET Network simplex
	% 4 CPX_ALG_BARRIER Barrier
	% 5 CPX_ALG_SIFTING Sifting
	% 6 CPX_ALG_CONCURRENT Concurrent (Dual, Barrier, and Primal)
	ops = cplexoptimset('lpmethod',0,'diagnostics','off'); % auto
	% problem with inequality constraints
	[sol,fval,exitflag] = cplexlp(cc,AA,bb,[],[],lb,ub,sol,ops);
end
time = toc;

if exitflag~=1
	fprintf('Optimization problem, exitflag=%i,\n',exitflag);
	% CPLEX outputs a variety of problems
	% check feasibility
	feas = max(0,max(AA*sol-bb));
	fprintf('Infeasibility: %f\n',feas);
	if feas>1e-3% this solution is not good
		error('Optimization problem, exitflag=%i,\n',exitflag);
	end
	if exitflag==-3
		% -3 means the problem unbounded
		% let's check it
		assert(abs(sol(const)-1)<1e-8) % meets constraints
		assert(abs(lb(const)-1)<1e-8) % constraint present
		assert(abs(ub(const)-1)<1e-8) % constraint present
		% if the above is satisfied then alpha*sol is not a solution
		% so it is bullshit that the problem is unbounded, just continue
	elseif exitflag==-2
		% this is bullshit too, we checked it is feasible and gave the
		% solver feasible solution
		% try the other solver
		fprintf('CPLEX reportet problem infeasible\n');
		ops = optimset('LargeScale','on','Display','iter');
		[sol,fval,exitflag] = linprog(cc,AA,bb,[],[],lb,ub,sol,ops);
		if exitflag<0
			error('Cannot solve LP, exitflag=%i,\n',exitflag);
		end
	else
		error('Optimization problem, exitflag=%i,\n',exitflag);
	end
end

% extract solution
Zeta = sol(zeta);
Xi = row(1-Zeta(2:nV+1)); % if Xi=1 then map to y
if 0 % debug check we get what we wanted
	% reparametrization
	Phi = sol(phi);
	f = E1.fA(H.A>0);
	df = - A0'*Phi; % reparametrization
	
	% construct matrix P_zeta 
	PZeta = zeros(1,nI);
	PZeta(1) = 1;
	ii = zeros(1,nA);
	ii(1) = 1;
	for a = 1:nA
		d = H.d(a);
		z = 2^d-1;
		mui = mu(z+1,a);
		ii(a) = mui;
		PZeta(mui) = sol(zeta(a));
	end
	PT = sparse(1:nI,1:nI,PZeta,nI,nI);
	% 
	II = speye(nI);
	% verify LP-improving
	g = (II-PT)*f;
	h = g+df;
	assert(all(h >=-1e-5));
	E2 = E1;
	E2.fA(H.A>0) = g;
	[x LB mu1] = E2.full_LP(false,LP);
	assert(LB>-1e-4);
	% ok if passed to here, P is relaxed-improving
	% remains to check P^2 is relaxed-improving
	% try composition
	g = (II-PT^100)*f;
	E2.fA(H.A>0) = g;
	[x LB] = E2.full_LP(false,LP);
	assert(LB>-1e-4);
end

%check integrality
nz = sum(abs(min(Xi,1-Xi))>1e-3);
%assert(nz==0);
if nz>0
	fprintf('non-integrality: %f\n',max(abs(min(Xi,1-Xi))));
	% try if this suspecious solution is going to pass
	fprintf('Infeasibility: %f\n',max(0,max(AA*sol-bb)));
	% if feasible, it is Ok to round down but may be non-optimal
	Xi = logical(Xi>1e-4);
	p = repmat([0;1],1,nV);
	p(:,Xi) = repmat(y(Xi),2,1);
	try 
		hpo_verify(F,p,LP);
	catch e
		% does not pass
		error('Bad optimization');
	end
	return;
end
Xi = logical(Xi>1e-3);
p = repmat([0;1],1,nV);
p(:,Xi) = repmat(y(Xi),2,1);

end