function [p time] = hpo_L1(E,y,LP)

nV = get_nV(E);
nA = get_nA(E);
H = E.H;
I = H.A;
nI = nnz(I);
I(I>0) = 1:nI;

if(~exist('y','var') || isempty(y))
	[y LB mu1 Mu] = E.full_LP(false,LP);
end

mask = y==1;
E1 = E.flip_energy(mask); % want that y becomes flipped to 0
E1 = E1.to_multilinear(); % reparametrize to a miltilinear polynomial (unique)
% now all components with at least one x_i=0 are zero
% create linear program
A0 = E1.get_LP(LP);
%nvars = size(A,2);
%const = 1;
mu = double(E.H.A>0);
n = nnz(mu);
mu(mu>0) = 1:n;
% A is my LP relaxation matrix
% enumerate variables in L1
i = 1;
n = size(A0,1);  phi = i:i+n-1; i = i+n; % dual multipliers <-> primal constraints
n = nV; xi = i:i+n-1; i = i+n; % indicator variables for the mapping to sent to 0
const = i; i = i+1; % constant term = 1
nvars = i-1;

% variable limits
lb = -inf(nvars,1);
ub = inf(nvars,1);
lb(xi) = 0;
ub(xi) = 1;
lb(const) = 1; ub(const) = 1;

%get vector f
f = E1.fA(H.A>0);
%g0 = zeros(size(f));

%dual representation
A = A0';
% extend the matrix for variables xi
A(:,nvars) = 0;
% matrix for extra constraints
A1 = sparse(0,size(A,2));
% generate constraints
eq = 0;
for a=1:nA % loop over components
	d = H.d(a);
	v = H.get_nodes(a);
	for z=0:2^d-1 % loop over all configurations
		if d>0 && z==2^d-1 % configuration 1_a on clique a
			if d==1 % vertices
				% get mu index for component a, configuration x_a = 1
				mui = mu(z+1,a);
				% constraint (A'\phi)_s(x_s) -f_s(1)xi_s <= 0
				% the part (A'\phi)_s(x_s) is already there, add -f_s(1)xi_s
				A(mui,xi(v)) = -f(mui);
			else % d>1, higher order term
				% get mu index for component a, configuration x_a = 1_a
				mui = mu(z+1,a);
				if(f(mui) <= 0)% case when f_a(1_a) < 0
					%constraint (A'\phi)_c(1_c) -f_c(1)xi_s <= 0 for all s\in a
					for s=v % all vertices in a
						if s==v(end) % write into existing constraint
							A(mui,xi(s)) = -f(mui);
						else
							% have to duplicate the constraint
							eq = eq+1;
							A1(eq,:) = A(mui,:);
							A1(eq,xi(s)) = -f(mui);
						end
					end
				else % case f_a(1_a) >= 0
					% first, duplicate constraint
					eq = eq+1;
					A1(eq,:) = A(mui,:);
					%constraint (A'\phi)_c(1_c) -f_c(1_c) sum_{s\in c}xi_s <= 0
					A(mui,xi(v)) = -f(mui);
					%constraint (A'\phi)_c(1_c) -f_c(1_c) <= 0
					A1(eq,const) = -f(mui);
				end
			end
		else
			%all other configurations, for them f_a(x_a) = 0
			%constraint (A'\phi)_a(x_a) <= 0
			% just do not append any vars and we are done
		end
	end
end
AA = [A; A1]; % concatenate all constraints
bb = zeros(size(AA,1),1);
% test program is feasible
sol = zeros(nvars,1);
sol(const) = 1;
assert(max(AA*sol-bb)<=0);

% objective
cc = zeros(1,nvars); cc(xi) = -1; % maximize \sum_s \xi_s
tic
if 0
	ops = cplexoptimset('lpmethod',0,'diagnostics','off','timelimit',1000); % automatic, 10 min time limit
	ctype = repmat('C',1,nvars);
	ctype(xi) = 'B';
	[sol,fval,exitflag] = cplexmilp(cc,AA,bb,[],[],[],[],[],lb,ub,ctype,[],ops);
else
	ops = cplexoptimset('lpmethod',0,'diagnostics','off'); % auto
	% problem with inequality constraints
	%[sol,fval,exitflag] = cplexlp(cc,AA,bb,[],[],lb,ub,sol,ops);
	% take a look at the reparametrization
	[sol,fval,exitflag] = linprog(cc,AA,bb,[],[],lb,ub,sol);
end
time = toc;
% extract solution
Xi = row(sol(xi)); % if Xi=1 then map to y
if 1 % check we get what we wanted
	% reparametrization
	Phi = sol(phi);
	f = E1.fA(H.A>0);
	df = - A0'*Phi; % reparametrization
	
	% construct zeta
	zeta = zeros(1,nI);
	zeta(1) = 1;
	ii = zeros(1,nA);
	ii(1) = 1;
	for a = find(H.d==1)
		d = H.d(a);
		v = H.get_nodes(a);
		z = 2^d-1;
		mui = mu(z+1,a);
		ii(a) = mui;
		zeta(mui) = 1-Xi(v);
	end
	
	for a=find(H.d>1)
		d = H.d(a);
		v = H.get_nodes(a);
		z = 2^d-1;
		mui = mu(z+1,a);
		ii(a) = mui;
		if(d==1)% node
			zeta(mui) = 1-Xi(v);
		else % higher order components
			if f(mui<=0) % applied constraint zeta  <= min(1-xi_s)
				zeta(mui) = min(1 - Xi(v));
			else % applied constraint zeta >= max(0,1-sum_x xi_s)
				zeta(mui) = max(0,1-sum(Xi(v)));
			end
		end
	end
	% check zeta satisfies constraints of Zeta
	for a = 2:nA
		d = H.d(a);
		z = 2^d-1;
		v = H.get_nodes(a);
		mui = mu(z+1,a);
		assert(zeta(mui) < 1+1e-5);
		assert(zeta(mui) > 0-1e-5);
		if d>1
			assert(zeta(mui) < min(zeta(ii(v+1)))+1e-5);
			assert(zeta(mui) > 1-sum(1-zeta(ii(v+1)))-1e-5);
		end
	end
	II = speye(nI);
	%PT = E.get_PTzeta(zeta);
	PT = sparse(1:nI,1:nI,zeta,nI,nI);
	% verify LP-improving
	g = (II-PT)*f;
	h = g+df;
	assert(all(h >=-1e-5));
	E2 = E1;
	E2.fA(H.A>0) = g;
	[x LB mu1] = E2.full_LP(false,LP);
	assert(LB>-1e-4);
end

if 0
	Phi = sol(phi);
	f = E1.fA(H.A>0);
	f = f - A0'*Phi; % reparametrized energy
	E2 = E1;
	E2.fA(H.A>0) = f;
end
if exitflag<0
	error('Optimization problem');
end
%check integrality
nz = sum(abs(min(Xi,1-Xi))>1e-3);
%assert(nz==0);
if nz>0
	fprintf('non-integrality: %f\n',max(abs(min(Xi,1-Xi))));
	error('bad');
end
Xi = logical(Xi>0.1);
p = repmat([0;1],1,nV);
p(:,Xi) = repmat(y(Xi),2,1);

end