function [p time] = hpo_swoboda(F,y,LP)
nV = get_nV(F);
nA = get_nA(F);
H = F.H;
I = H.A;
nI = nnz(I);
I(I>0) = 1:nI;

if(~exist('y','var') || isempty(y))
	[y LB mu1 Mu] = F.full_LP(false,LP);
end

p = repmat(y,2,1); % all to y

G = F;
f = F.fA(F.H.A>0);

A = true(1,nV);

time = 0;

it = 1;
while true
	fprintf('it %i, ',it);
	% verification LP
	% construct auxiliary energy function according to Swoboda et al. 2014
	% using current set A
	for a = 2:nA % for every hyperedge
		v = row(H.get_nodes(a));
		if all(A(v))% fully inside, keep unchanged
			G.fA(:,a) = F.fA(:,a);
		elseif all(~A(v)) % fully outside, zero out
			G.fA(:,a) = 0;
		else % boundary hyperedge, do the max-min stuff
			mask = A(v);
			% first variables goes as lower bits
			mask = fliplr(mask);
			n1 = nnz(mask);
			n2 = nnz(~mask);
			for z1 = 0:2^n1-1 % loop over inner configurations
				f_min = inf;
				f_max = -inf;
				x1 = mdec2bin(z1,n1);
				% assignment y on the interior
				sy = fliplr(y(v(A(v)))); % first vertices go to lower bits
				for loop=1:2 % first pass calculate min and max, second assigns
					for z2 = 0:2^n2-1 % loop over outer configurations
						x2 = mdec2bin(z2,n2);
						% concatenated configuration
						x = repmat(0,1,n1+n2);
						x(mask) = x1;
						x(~mask) = x2;
						z = mbin2dec(x);
						if loop==1
							fz = F.fA(z+1,a);
							if fz<f_min
								f_min = fz;
							end
							if fz>f_max
								f_max = fz;
							end
						else
							if all(x1==sy) % condition x_{A \cap e} = y_{A \cap e}
								G.fA(z+1,a) = f_max;
							else
								G.fA(z+1,a) = f_min; % this if for all except when x1 = y_{e\cap A}
							end
						end
					end
				end
			end
		end
	end
	
	[x LB mu1 Mu ans ans dt] = G.full_LP(false,LP);
	time = time+dt;
	fprintf('verification LP: %f\n',LB);

	nint = sum(mu1.^2,1)<1-1e-3; % non-integer pixels
	W = mselect(mu1,y+1,1:nV)<1-1e-4;% find where mu disagrees with y
	assert(all(W >= nint));
	% want to have A & nint = 0 and A & W = 0
	if ~any(A & W) % current map is optimal
		return;
	end
	% remove some labels from the map
	for v=find(A & W) %1:nV
		%if mu1(y(v)+1,v) < 1-1e-5 % optimal solution selects something else that y
		p(:,v) = [0;1]; % immovable
		A(v) = false;
		%end
	end
	it = it+1;
end

end