function [elim p time ilp_time ilp_reduced_time] = invoke_pho_method(method,E,y)

K = 2;
nV = E.get_nV();

if strcmp('L1-FLP',method)
	try
		[p time] = hpo_L1(E,y,'FLP');
	catch e % LP was too difficult, solve by the cutting plane method
		display(e.message);
		fprintf('Resorting to iterative algorithm\n');
		[p time] = hpo_iterative(E,y,'FLP');
	end
	[ilp_time ilp_reduced_time] = hpo_verify(E,p,'FLP');
end

if strcmp('L1-Iterative-FLP',method)
	[p time] = hpo_iterative(E,y,'FLP');
	[ilp_time ilp_reduced_time] = hpo_verify(E,p,'FLP');
end

if strcmp('L1-Iterative-BLP',method)
	% do progressively, as BLP refines with fixation
	E1 = E;
	y1 = y;
	p = repmat([0;1],1,nV);
	ppi = 1:nV; % aggregated map
	time = 0;
	while true
		[p1 time1] = hpo_iterative(E1,y1,'BLP');
		m = p1(1,:)==p1(2,:);
		e = sum(m);
		fprintf('elim: %i\n',e);
		time = time+time1;
		hpo_verify(E1,p1,'BLP');
		if all(p1(1,:)~=p1(2,:)) % nothing more to reduce
			break;
		end
		p(:,ppi) = p1; % aggregate reduction		
		[E2 pi] = E1.restrict(p1);
		% get a new test labeling, can be better, because reduction
		assert(E1(y1) == E2(y1(pi))); % check restriction works
		x = randi(2,1,length(y1))-1;
		x(m) = y1(m);
		assert(E1(x) == E2(x(pi))); % check restriction works
		if(E1.get_nV()==0) % problem is constant
			break;
		end		
		E1 = E2;
		% tightened BLP
		y1 = E1.full_LP(false,'BLP');
		ppi = ppi(pi);
	end
	% resulting map must be FLP-improving
	[ilp_time ilp_reduced_time] = hpo_verify(E,p,'FLP');
end

if strcmp('Swoboda-14',method)
	[p time] = hpo_swoboda(E,y,'FLP');
	[ilp_time ilp_reduced_time] = hpo_verify(E,p,'FLP');
end

if ~isempty(strmatch(method,{'HOCR','Fix','GRD-heur','GRD','GRD-gen'}))
	[p time] = hpo_grd(E,y,method);
	%if  %~isempty(strmatch(method,{'GRD-heur','GRD','GRD-gen','HOCR'}))
	[ilp_time ilp_reduced_time] = hpo_verify(E,p,'FLP');
	%else
	%	hpo_verify(E,p,'BLP');
	%end
	%hpo_verify(E,p,'FLP');
end
p = double(p);
alive = sum(all(p==repmat([0;1],1,nV),1));
%elim = (nV-alive)/nV*100;
elim = nV-alive;
fprintf('eliminated: %i  time: %3.2fs\n',elim,time);

end